/*
 * Realtek RTL2832 demod driver
 *
 *    Copyright (c) 2008 Realtek
 *    Copyright (c) 2008 Jan Hoogenraad, Barnaby Shearer, Andy Hasper
 *    Copyright (C) 2009 Antti Palosaari <crope@iki.fi>
 *    Copyright (c) 2011 Maxim Levitsky <maximlevitsky@gmail.com>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef RTL2332_H
#define RTL2332_H

#include <linux/i2c.h>


#define REG(page, address, msb, lsb) \
	((page) | (address << 8) | (msb << 16) | (lsb << 24))

/* ========================== page 0 ============================ */
#define DVBT_OPT_ADC_IQ		REG(0x0, 0x06,	5,	4)
#define DVBT_MPEG_IO_OPT_2_2	REG(0x0, 0x06,	7,	7)
#define DVBT_MPEG_IO_OPT_1_0	REG(0x0, 0x07,	7,	6)
#define DVBT_AD_EN_REG1		REG(0x0, 0x08,	6,	6)
#define DVBT_AD_EN_REG		REG(0x0, 0x08,	7,	7)
#define DVBT_AD_AVI		REG(0x0, 0x09,	1,	0)
#define DVBT_AD_AVQ		REG(0x0, 0x09,	3,	2)
#define DVBT_AD_AV_REF		REG(0x0, 0x09,	6,	0)
#define DVBT_REG_PI		REG(0x0, 0x0a,	2,	0)
#define DVBT_REG_MON		REG(0x0, 0x0d,	1,	0)
#define DVBT_REG_MONSEL		REG(0x0, 0x0d,	2,	2)
#define DVBT_REG_GPE		REG(0x0, 0x0d,	7,	7)
#define DVBT_POLAR_RF_AGC	REG(0x0, 0x0e,	1,	1)
#define DVBT_POLAR_IF_AGC	REG(0x0, 0x0e,	0,	0)
#define DVBT_REG_GPO		REG(0x0, 0x10,	0,	0)
#define DVBT_AD7_SETTING	REG(0x0, 0x11,	15,	0)
#define DVBT_REG_4MSEL		REG(0x0, 0x13,	0,	0)
#define DVBT_PIP_ON		REG(0x0, 0x21,	3,	3)
#define DVBT_VAL_LVL_PIP	REG(0x0, 0xb7,	0,	0)
#define DVBT_ERR_LVL_PIP	REG(0x0, 0xb7,	1,	1)
#define DVBT_SYNC_LVL_PIP	REG(0x0, 0xb7,	2,	2)
#define DVBT_CKOUT_PWR_PIP	REG(0x0, 0xb7,	3,	3)
#define DVBT_CKOUTPAR_PIP	REG(0x0, 0xb7,	4,	4)

#define DVBT_VAL_LVL_PID	REG(0x0, 0xb9,	0,	0)
#define DVBT_ERR_LVL_PID	REG(0x0, 0xb9,	1,	1)
#define DVBT_SYNC_LVL_PID	REG(0x0, 0xb9,	2,	2)
#define DVBT_CKOUT_PWR_PID	REG(0x0, 0xb9,	3,	3)
#define DVBT_CKOUTPAR_PID	REG(0x0, 0xb9,	4,	4)

/* ========================== page 1 ============================ */
#define DVBT_SOFT_RST		REG(0x1, 0x01, 	2,	2)
#define DVBT_IIC_REPEAT		REG(0x1, 0x01,	3,	3)
#define DVBT_AGC_TARG_VAL_0	REG(0x1, 0x02,	0,	0)
#define DVBT_AGC_TARG_VAL_8_1	REG(0x1, 0x03,	7,	0)
#define DVBT_LOOP_GAIN2_3_0	REG(0x1, 0x04,	4,	1)
#define DVBT_AAGC_HOLD		REG(0x1, 0x04,	5,	5)
#define DVBT_EN_RF_AGC		REG(0x1, 0x04,	6,	6)
#define DVBT_EN_IF_AGC		REG(0x1, 0x04,	7,	7)
#define DVBT_LOOP_GAIN2_4	REG(0x1, 0x05,	7,	7)
#define DVBT_KRF2		REG(0x1, 0x07,	7,	0)

#define DVBT_IF_AGC_MIN		REG(0x1, 0x08,	7,	0)
#define DVBT_IF_AGC_MAX		REG(0x1, 0x09,	7,	0)
#define DVBT_RF_AGC_MIN		REG(0x1, 0x0a,	7,	0)
#define DVBT_RF_AGC_MAX		REG(0x1, 0x0b,	7,	0)

#define DVBT_IF_AGC_MAN		REG(0x1, 0x0c,	6,	6)
#define DVBT_IF_AGC_MAN_VAL	REG(0x1, 0x0c,	13,	0)
#define DVBT_RF_AGC_MAN		REG(0x1, 0x0e,	6,	6)
#define DVBT_RF_AGC_MAN_VAL	REG(0x1, 0x0e,	13,	0)
#define DVBT_DAGC_TRG_VAL	REG(0x1, 0x12,	7,	0)
#define DVBT_SPEC_INV		REG(0x1, 0x15,	0,	0)
#define DVBT_PSET_IFFREQ	REG(0x1, 0x19,	21,	0)

#define DVBT_EN_CACQ_NOTCH	REG(0x1, 0x61,	4,	4)
#define DVBT_KB_P1		REG(0x1, 0x64,	3,	1)
#define DVBT_KB_P2		REG(0x1, 0x64,	6,	4)
#define DVBT_KB_P3		REG(0x1, 0x65,	2,	0)

#define DVBT_TRK_KS_P2		REG(0x1, 0x6f,	2,	0)
#define DVBT_TRK_KS_I2		REG(0x1, 0x70,	5,	3)
#define DVBT_TR_THD_SET2	REG(0x1, 0x72,	3,	0)
#define DVBT_TRK_KC_P2		REG(0x1, 0x73,	5,	3)
#define DVBT_TRK_KC_I2		REG(0x1, 0x75,	2,	0)
#define DVBT_CR_THD_SET2	REG(0x1, 0x76,	7,	6)

#define DVBT_CKOUTPAR		REG(0x1, 0x7b,	5,	5)
#define DVBT_CKOUT_PWR		REG(0x1, 0x7b,	6,	6)
#define DVBT_SYNC_DUR		REG(0x1, 0x7b,	7,	7)
#define DVBT_ERR_DUR		REG(0x1, 0x7c,	0,	0)
#define DVBT_SYNC_LVL		REG(0x1, 0x7c,	1,	1)
#define DVBT_ERR_LVL		REG(0x1, 0x7c,	2,	2)
#define DVBT_VAL_LVL		REG(0x1, 0x7c,	3,	3)
#define DVBT_SERIAL		REG(0x1, 0x7c,	4,	4)
#define DVBT_SER_LSB		REG(0x1, 0x7c,	5,	5)
#define DVBT_CDIV_PH0		REG(0x1, 0x7d,	3,	0)
#define DVBT_CDIV_PH1		REG(0x1, 0x7d,	7,	4)

#define DVBT_TR_WAIT_MIN_8K	REG(0x1, 0x88,	11,	2)
#define DVBT_RSD_BER_FAIL_VAL	REG(0x1, 0x8f,	15,	0)

#define DVBT_SM_PASS		REG(0x1, 0x93,	11,	0)

#define DVBT_MGD_THD0		REG(0x1, 0x95,	7,	0)
#define DVBT_MGD_THD1		REG(0x1, 0x96,	7,	0)
#define DVBT_MGD_THD2		REG(0x1, 0x97,	7,	0)
#define DVBT_MGD_THD3		REG(0x1, 0x98,	7,	0)
#define DVBT_MGD_THD4		REG(0x1, 0x99,	7,	0)
#define DVBT_MGD_THD5		REG(0x1, 0x9a,	7,	0)
#define DVBT_MGD_THD6		REG(0x1, 0x9b,	7,	0)
#define DVBT_MGD_THD7		REG(0x1, 0x9c,	7,	0)

#define DVBT_CFREQ_OFF_RATIO	REG(0x1, 0x9d,	23,	4)
#define DVBT_RSAMP_RATIO	REG(0x1, 0x9f,	27,	2)

#define DVBT_EN_BK_TRK		REG(0x1, 0xa6,	7,	7)
#define DVBT_EN_BBIN		REG(0x1, 0xb1,	0,	0)

#define DVBT_AAGC_LOOP_GAIN	REG(0x1, 0xc7,	5,	1)

#define DVBT_LOOP_GAIN3		REG(0x1, 0xc8,	4,	0)
#define DVBT_VTOP1		REG(0x1, 0x06,	5,	0)
#define DVBT_VTOP2		REG(0x1, 0xc9,	5,	0)
#define DVBT_VTOP3		REG(0x1, 0xca,	5,	0)
#define DVBT_KRF1		REG(0x1, 0xcb,	7,	0)
#define DVBT_KRF3		REG(0x1, 0xcd,	7,	0)
#define DVBT_KRF4		REG(0x1, 0xce,	7,	0)

#define DVBT_EN_GI_PGA		REG(0x1, 0xe5,	0,	0)
#define DVBT_THD_LOCK_UP	REG(0x1, 0xd9,	8,	0)
#define DVBT_THD_LOCK_DW	REG(0x1, 0xdb,	8,	0)
#define DVBT_THD_UP1		REG(0x1, 0xdd,	7,	0)
#define DVBT_THD_DW1		REG(0x1, 0xde,	7,	0)
#define DVBT_EN_AGC_PGA		REG(0x1, 0xd7,	0,	0)
#define DVBT_INTER_CNT_LEN	REG(0x1, 0xd8,	3,	0)
#define DVBT_GI_PGA_STATE	REG(0x1, 0xe6,	3,	3)


/* ========================== page 2 ============================ */

#define DVBT_SCALE1_B92		REG(0x2, 0x92,	7,	0)
#define DVBT_SCALE1_B93		REG(0x2, 0x93,	7,	0)
#define DVBT_SCALE1_BA7		REG(0x2, 0xa7,	7,	0)
#define DVBT_SCALE1_BA9		REG(0x2, 0xa9,	7,	0)
#define DVBT_SCALE1_BAA		REG(0x2, 0xaa,	7,	0)
#define DVBT_SCALE1_BAB		REG(0x2, 0xab,	7,	0)
#define DVBT_SCALE1_BAC		REG(0x2, 0xac,	7,	0)
#define DVBT_SCALE1_BB0		REG(0x2, 0xb0,	7,	0)
#define DVBT_SCALE1_BB1		REG(0x2, 0xb1,	7,	0)
#define DVBT_K1_CR_STEP12	REG(0x2, 0xad,	9,	4)

/* ========================== page 3 ============================ */

#define DVBT_RSSI_R		REG(0x3, 0x01,	6,	0)
#define DVBT_DAGC_VAL		REG(0x3, 0x05,	7,	0)
#define DVBT_ACI_DET_IND	REG(0x3, 0x12,	0,	0)
#define DVBT_SFREQ_OFF		REG(0x3, 0x18,	13,	0)
#define DVBT_RX_CONSTEL		REG(0x3, 0x3c,	3,	2)
#define DVBT_RX_HIER		REG(0x3, 0x3c,	6,	4)
#define DVBT_RX_C_RATE_LP	REG(0x3, 0x3d,	2,	0)
#define DVBT_RX_C_RATE_HP	REG(0x3, 0x3d,	5,	3)
#define DVBT_RSD_BER_EST	REG(0x3, 0x4e,	15,	0)
#define DVBT_GI_IDX		REG(0x3, 0x51,	1,	0)
#define DVBT_FFT_MODE_IDX	REG(0x3, 0x51,	2,	2)
#define DVBT_FSM_STAGE		REG(0x3, 0x51,	6,	3)
#define DVBT_RF_AGC_VAL		REG(0x3, 0x5b,	13,	0)
#define DVBT_IF_AGC_VAL		REG(0x3, 0x59,	13,	0)
#define DVBT_CFREQ_OFF		REG(0x3, 0x5f,	17,	0)


/* ========================== page 4 ============================ */
#define DVBT_CE_EST_EVM		REG(0x4, 0xc,	15,	0)


/* only RTL2832_TUNER_FC0012 is supported by now */
enum rtl2832_tuner_type {
	RTL2832_TUNER_UNSET = 0,
	RTL2832_TUNER_UNKNOWN,
	RTL2832_TUNER_E4000,
	RTL2832_TUNER_FC0012,
	RTL2832_TUNER_FC2580,
	RTL2832_TUNER_MAX3543,
	RTL2832_TUNER_MT2063,
	RTL2832_TUNER_MT2266,
	RTL2832_TUNER_MXL5007T,
	RTL2832_TUNER_TUA9001,
	RTL2832_TUNER_LISTSIZE,
};

struct rtl2832_config {

	/* demod i2c address */
	u8 i2c_address;

	/* tuner type to fine-tune settings */
	enum rtl2832_tuner_type tuner_type;

	/* crystal frequency */
	unsigned long crystal_freq;

	/* true = parallel iface, false == serial */
	/* 	up to now all rtl2832
		devices use parallel mode, because they are
		integrated in the bridge, rtl2836/40 on the other hand
		use serial mode only*/
	bool ts_parallel;

	/* originally called func1, the code behind it seems
		to work around some device bug by resetting the device
		as specific condition triggers */
	bool periodic_update_hack;

};

#define MASK(bit) ((bit) < 0 ? 0 : (((1 << (bit)) - 1) | (1 << (bit))))

struct rtl2832_register_table_entry {
	u32 reg;
	u32 value;
};


/* TODO: at module separation add dummy version for disabled demod case */
struct dvb_frontend *rtl2832_attach(struct i2c_adapter *i2c);

int rtl2832_set_config(struct dvb_frontend *fe,
					struct rtl2832_config *config);

/* HACK: this hack will go away in the future */

#ifndef DVB_USB_LOG_PREFIX

extern int r2832u_debug;

#define DVB_USB_LOG_PREFIX "rtl2832"
#include "dvb-usb.h"


#define INFO_DEBUG 0x01
#define REG_DEBUG  0x02
#define I2C_DEBUG  0x04
#define XFER_DEBUG 0x08


#define deb_info(args...)   dprintk(r2832u_debug, INFO_DEBUG, args)
#define deb_reg(args...)    dprintk(r2832u_debug, REG_DEBUG, args)
#define deb_i2c(args...)    dprintk(r2832u_debug, I2C_DEBUG, args)
#define deb_xfer(args...)   dprintk(r2832u_debug, XFER_DEBUG, args)

#endif

#endif