/*
 * Realtek RTL2832 demod driver
 *
 * Copyright (c) 2008 Realtek
 * Copyright (c) 2008 Jan Hoogenraad, Barnaby Shearer, Andy Hasper
 * Copyright (C) 2009 Antti Palosaari <crope@iki.fi>
 * Copyright (c) 2011 Maxim Levitsky <maximlevitsky@gmail.com>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Common register settings */
/* Thats file speaks itself about why it sucks not not have a datasheet :-) */
/* This is of course not firmware, but just set of device knobs */

#include <linux/kernel.h>
#include "rtl2832.h"

struct rtl2832_register_table_entry rtl2832_common_regtable[] = {
	{DVBT_AD_EN_REG,		0x1	},
	{DVBT_AD_EN_REG1,		0x1	},
	{DVBT_RSD_BER_FAIL_VAL,		0x2800	},
	{DVBT_MGD_THD0,			0x10	},
	{DVBT_MGD_THD1,			0x20	},
	{DVBT_MGD_THD2,			0x20	},
	{DVBT_MGD_THD3,			0x40	},
	{DVBT_MGD_THD4,			0x22	},
	{DVBT_MGD_THD5,			0x32	},
	{DVBT_MGD_THD6,			0x37	},
	{DVBT_MGD_THD7,			0x39	},
	{DVBT_EN_BK_TRK,		0x0	},
	{DVBT_EN_CACQ_NOTCH,		0x0	},
	{DVBT_AD_AV_REF,		0x2a	},
	{DVBT_REG_PI,			0x6	},
	{DVBT_PIP_ON,			0x0	},
	{DVBT_CDIV_PH0,			0x8	},
	{DVBT_CDIV_PH1,			0x8	},
	{DVBT_SCALE1_B92,		0x4	},
	{DVBT_SCALE1_B93,		0xb0	},
	{DVBT_SCALE1_BA7,		0x78	},
	{DVBT_SCALE1_BA9,		0x28	},
	{DVBT_SCALE1_BAA,		0x59	},
	{DVBT_SCALE1_BAB,		0x83	},
	{DVBT_SCALE1_BAC,		0xd4	},
	{DVBT_SCALE1_BB0,		0x65	},
	{DVBT_SCALE1_BB1,		0x43	},
	{DVBT_KB_P1,			0x1	},
	{DVBT_KB_P2,			0x4	},
	{DVBT_KB_P3,			0x7	},
	{DVBT_K1_CR_STEP12,		0xa	},
	{DVBT_REG_GPE,			0x1	},
	{DVBT_TRK_KS_P2,		0x4	},
	{DVBT_TRK_KS_I2,		0x7,	},
	{DVBT_TR_THD_SET2,		0x6,	},
	{DVBT_TRK_KC_I2,		0x5,	}, /* 6 for STB mode */
	{DVBT_CR_THD_SET2,		0x1,	},
	{0,				0x0,	},
};

/* registers for parallel TS interface */
struct rtl2832_register_table_entry rtl2832_ts_parallel_regtable[] = {
	{DVBT_SERIAL,			0x0	},
	{DVBT_CDIV_PH0,			0x9	},
	{DVBT_CDIV_PH1,			0x9	},
	{DVBT_MPEG_IO_OPT_2_2,		0x0	},
	{DVBT_MPEG_IO_OPT_1_0,		0x0	},
	{0,				0x0,	},
};

/* registers for serial TS interface */
struct rtl2832_register_table_entry rtl2832_ts_serial_regtable[] = {
	{DVBT_SERIAL,			0x1	},
	{DVBT_CDIV_PH0,			0x1	},
	{DVBT_CDIV_PH1,			0x2	},
	{DVBT_MPEG_IO_OPT_2_2,		0x0	},
	{DVBT_MPEG_IO_OPT_1_0,		0x1	},
	{0,				0x0,	},
};

/* Specific registers for E4000 tuner */
static const struct rtl2832_register_table_entry rtl2832_e4000_regtable[] = {
	{DVBT_DAGC_TRG_VAL,		0x5a	},
	{DVBT_AGC_TARG_VAL_0,		0x0	},
	{DVBT_AGC_TARG_VAL_8_1,		0x5a	},
	{DVBT_AAGC_LOOP_GAIN,		0x18    },
	{DVBT_LOOP_GAIN2_3_0,		0x8	},
	{DVBT_LOOP_GAIN2_4,		0x1	},
	{DVBT_LOOP_GAIN3,		0x18	},

	{DVBT_VTOP1,			0x35	},
	{DVBT_VTOP2,			0x21	},
	{DVBT_VTOP3,			0x21	},
	{DVBT_KRF1,			0x0	},
	{DVBT_KRF2,			0x40	},
	{DVBT_KRF3,			0x10	},
	{DVBT_KRF4,			0x10	},
	{DVBT_IF_AGC_MIN,		0x80	},
	{DVBT_IF_AGC_MAX,		0x7f	},
	{DVBT_RF_AGC_MIN,		0x80	},
	{DVBT_RF_AGC_MAX,		0x7f	},
	{DVBT_POLAR_RF_AGC,		0x0	},
	{DVBT_POLAR_IF_AGC,		0x0	},
	{DVBT_AD7_SETTING,		0xe9d4	},
	{DVBT_EN_GI_PGA,		0x0	},
	{DVBT_THD_LOCK_UP,		0x0	},
	{DVBT_THD_LOCK_DW,		0x0	},
	{DVBT_THD_UP1,			0x14	},
	{DVBT_THD_DW1,			0xec	},

	{DVBT_INTER_CNT_LEN,		0xc	},
	{DVBT_GI_PGA_STATE,		0x0	},
	{DVBT_EN_AGC_PGA,		0x1	},

	{DVBT_REG_GPE,			0x1	},
	{DVBT_REG_GPO,			0x1	},
	{DVBT_REG_MONSEL,		0x1	},
	{DVBT_REG_MON,			0x1	},
	{DVBT_REG_4MSEL,		0x0	},
	{0,				0x0,	},
};

/* Specific registers for FC0012 tuner */
static const struct rtl2832_register_table_entry rtl2832_fc0012_regtable[] = {
	{DVBT_DAGC_TRG_VAL,		0x5a	},
	{DVBT_AGC_TARG_VAL_0,		0x0	},
	{DVBT_AGC_TARG_VAL_8_1,		0x5a	},
	{DVBT_AAGC_LOOP_GAIN,		0x16    },
	{DVBT_LOOP_GAIN2_3_0,		0x6	},
	{DVBT_LOOP_GAIN2_4,		0x1	},
	{DVBT_LOOP_GAIN3,		0x16	},
	{DVBT_VTOP1,			0x35	},
	{DVBT_VTOP2,			0x21	},
	{DVBT_VTOP3,			0x21	},
	{DVBT_KRF1,			0x0	},
	{DVBT_KRF2,			0x40	},
	{DVBT_KRF3,			0x10	},
	{DVBT_KRF4,			0x10	},
	{DVBT_IF_AGC_MIN,		0x80	},
	{DVBT_IF_AGC_MAX,		0x7f	},
	{DVBT_RF_AGC_MIN,		0x80	},
	{DVBT_RF_AGC_MAX,		0x7f	},
	{DVBT_POLAR_RF_AGC,		0x0	},
	{DVBT_POLAR_IF_AGC,		0x0	},
	{DVBT_AD7_SETTING,		0xe9bf	},
	{DVBT_EN_GI_PGA,		0x0	},
	{DVBT_THD_LOCK_UP,		0x0	},
	{DVBT_THD_LOCK_DW,		0x0	},
	{DVBT_THD_UP1,			0x11	},
	{DVBT_THD_DW1,			0xef	},
	{DVBT_INTER_CNT_LEN,		0xc	},
	{DVBT_GI_PGA_STATE,		0x0	},
	{DVBT_EN_AGC_PGA,		0x1	},
//	{DVBT_REG_GPE,			0x1	},
//	{DVBT_REG_GPO,			0x0	},
//	{DVBT_REG_MONSEL,		0x0	},
//	{DVBT_REG_MON,			0x3	},
//	{DVBT_REG_4MSEL,		0x0	},
	{0,				0x0,	},
};


const struct rtl2832_register_table_entry*
	tuner_register_tables[RTL2832_TUNER_LISTSIZE] = {

	[RTL2832_TUNER_E4000] = rtl2832_e4000_regtable,
	[RTL2832_TUNER_FC0012] = rtl2832_fc0012_regtable,
};
