/*
 * Realtek RTL28XX DVB USB driver
 *
 * Copyright (c) 2008 Realtek
 * Copyright (c) 2008 Jan Hoogenraad, Barnaby Shearer, Andy Hasper
 * Copyright (C) 2009 Antti Palosaari <crope@iki.fi>
 * Copyright (c) 2011 Maxim Levitsky <maximlevitsky@gmail.com>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Thanks to Realtek for a lot of support we received !
 */

#ifndef RTL2831U_H
#define RTL2831U_H

#define DVB_USB_LOG_PREFIX "r2832u"
#include "dvb-usb.h"


/* Firmware interface */
#define CTL_REG			0x0100
#define CTL_I2C			0x0300
#define CTL_I2C_RAW		0x0600

#define CTL_REG_SKIP_BYTESWAP	0x0001
#define CTL_WRITE		0x0010


/* USB System Control Register */
#define USB_SYSCTL         0x2000
			/* 0x00000400	fw: set, then cleared in reset situations */
			/* 0x00000001	fw/drv: always set, probably a enable bit */
			/* 0x00000008   drv:: FULL packet mode */

/* Interrupt status & enable (SIE) */
#define USB_IRQSTAT        0x2008
#define USB_IRQEN          0x200C

/* USB Control Register */
#define USB_CTRL           0x2010
			  /*  0x00000001  might be enable/disable bit */
			  /*  0x00000004  remote wakeup*/
			  /*  0x00000008  set on interrupt bit 2 */
			  /*  0x00000010  set and cleared on interrupt bit 2 */
			  /*  0x00000020  set iff RAM_FLAG2 is set - BTN_WAKEUP_DIS*/

/* USB Status Register */
#define USB_STAT           0x2014
			  /*  0x00000001  1 - high speed, 0 - full speed*/

/* USB Device Address */
#define USB_DEVADDR        0x2018
			  /* 16-bit device address */

/* USB Test Mode Register */
#define USB_TEST           0x201C
			  /* 0x00000002 - Test_J
			     0x00000003 - Test_K
			     0x00000001 - Test_SE0_NAK
			     0x00000004 - Test_Packet (send standard control response with test packet, then set that value
			  */


/* USB Frame number */
#define USB_FRAME_NUMBER   0x2020
			  /* not used */

/* Address of SIE FIFO RAM*/
#define USB_FIFO_ADDR      0x2028
			  /* set to 1 _before_ fifo memory read */
			  /* set to 2 _after_ fifo memory write */

/* SIE FIFO RAM Access Command*/
#define USB_FIFO_CMD       0x202A
			  /* - 0 is written here before fifo access - maybe that endpoint number?*/

/* SIE FIFO RAM Data Register!*/
#define USB_FIFO_DATA      0x2030
			  /* register is sort of exception, it
			   appears to be 'little-endian'.
			   the 0x2030 is first byte to send/receive,
			   the 0x2031 is second, etc...
			   allows to access the command FIFO*/


/* Control endpoint setup packet, broken into bytes for ease of use - but is big endian! =====*/
#define USB_SETUP_REQUEST_TYPE	0x20FB /* byte 0 */
#define USB_SETUP_REQUEST	0x20FA /* byte 1 */
#define USB_SETUP_VALUE_LO	0x20F9 /* byte 2 */
#define USB_SETUP_VALUE_HI	0x20F8 /* byte 3 */
#define USB_SETUP_INDEX_LO	0x20FF /* byte 4 */
#define USB_SETUP_INDEX_HI	0x20FE /* byte 5 */
#define USB_SETUP_LEN_LO	0x20FD /* byte 6 */
#define USB_SETUP_LEN_HI	0x20FC /* byte 7 */


/* Control endpoint configure Register */
#define USB_EP0_CONFIG		0x2104
				/* 0x00000080  only this bit is set. must be a enable bit */

/* Control endpoint control Register */
#define USB_EP0_CONTROL		0x2108
				/* fw: 0x00000001 - send OK */
				/* fw: 0x00000002 - receive OK */
				/* fw: 0x00000010 - stall */
				/* fw: 0x00000020 - flush/reset possibly */

/* Control endpoint status Register */
#define USB_EP0_STATUS		0x210C
				/* fw: 0x00000080 - device idle?? */
				/* fw: 0x00000002 - transmit fifo idle ??? */

/* Control endpoint interrupt Status Register */
#define USB_EP0_IRQ_STATUS	0x2110
				/* only init'ed to 0xFFFFFFFF */

/* Control endpoint interrupt Enable Register */
#define USB_EP0_IRQEN		0x2114
				/* not used */

/* Control endpoint max Packet Size Register */
#define USB_EP0_MAX_PACKET	0x2118
				/* maximum packet size - set to 0x40 - would you know!!!*/

/* Control endpoint FIFO Byte Counter Register */
#define USB_EP0_FIFO_BYTE_COUNT	0x2120
				/* number of bytes to send / receive - ored with 0x8000 for execution */

/* Configure Register */
#define USB_EPA_CFG		0x2144
				//3check epa config,
				//3[9-8]:00, 1 transaction per microframe
				//3[7]:1, epa enable
				//3[6-5]:10, bulk mode
				//3[4]:1, device to host
				//3[3:0]:0001, endpoint number

/* Bulk endpoint control Register */
#define USB_EPA_CTL		0x2148
				/* 0x00000200 - driver:  reset */
				/* 0x00000010 - driver : stall */

/* Bulk endpoint status Register  */
#define USB_EPA_STAT		0x214C

/* Bulk endpoint interrupt Status Register */
#define USB_EPA_IRQSTAT		0x2150

/* Bulk endpoint interrupt Enable Register */
#define USB_EPA_IRQEN		0x2154

/* Bulk endpoint max Packet Size Register */
#define USB_EPA_MAXPKT		0x2158

/* Bulk endpoint FIFO Configure Register*/
#define USB_EPA_FIFO_CFG	0x2160
				/* set to 0x14 - seems to be fifo length */


/* Demod control */
#define SYS_DEMOD_CTL		0x3003 /* Control register for DVB-T demodulator */
				/* 0x00000008 - ADC_Q - turn on to start demod */
				/* 0x00000020 - demod reset - set to 0 then  to 1 for a reset (after reset wait 5 ms)*/
				/* 0x00000040 - ADC_I - turn on to start demod */
				/* 0x00000080 - PLL setting, turn on to start demod */

/* RC control */
#define SYS_DEMOD_CTL1		0x3008 /* RC related settings */
				/* 0x00000004 - "IR_suspend" - turn on suspend */
				/* 0x00000008 - "Clk_400k" - turn on suspend */
				/* 0x00000010 - "suspend_latch", turn on suspend */


/* Gpio */
#define SYS_GPIO_OUTPUT_VALUE	0x3002 /* Output value of general purpose I/O */
#define SYS_GPIO_OUTPUT_ENABLE	0x3000 /* Output enable of general purpose I/O */
#define SYS_GPIO_DIRECTION	0x3007 /* Direction control for general purpose I/O */

#define SYS_GPIO_INPUT_VALUE	0x3001 /* Input value of general purpose I/O */

#define SYS_GPIO_PAD0		0x3004 /* PAD Configuration for GPIO0-GPIO3 */
#define SYS_GPIO_PAD1		0x300b /* PAD Configuration for GPIO4-GPIO7 */

				/* GPIO3 - output for tuner power
				 * input for some sort of 'Button  Wake UP'
				 * GPIO4 - output for tuner power
				 * for my tuner:
				 * GPIO5 - tuner reset
				 * GPIO6 - band
				 */


/* System Interrupt Enable/Status Register */
#define SYS_INT_ENABLE		0x3006
#define SYS_INT_STATUS		0x3005



/* I2C registers */
#define I2C_CLOCK		0x3040 /* I2C clock register */
#define I2C_MASTER_CONTROL	0x3044 /* I2C master control register */
#define I2C_MASTER_SCL		0x3048 /* I2C master SCL timing register */
#define I2C_MASTER_STATUS	0x304C /* I2C master status register */
#define I2C_MASTER_FIFO		0x3050 /* I2C master FIFO PIO access window */

/* IR registers */
#define IR_RX_BUF		0xFC00 /* 256 bytes RX buffer */

#define IR_RX_IE		0xFD00
#define IR_RX_IF		0xFD01

#define IR_RX_CTRL		0xFD02
#define IR_RX_CONFIG		0xFD03

#define IR_MAX_DURATION0	0xFD04
#define IR_MAX_DURATION1	0xFD05

#define IR_IDLE_LEN0		0xFD06
#define IR_IDLE_LEN1		0xFD07

#define IR_GLITCH_LEN		0xFD08

#define IR_RX_BUFFER_CTRL	0xFD09
#define IR_RX_BUFFER_DATA	0xFD0A

#define IR_RX_BC		0xFD0B
#define IR_RX_CLK		0xFD0C

#define IR_RX_C_COUNT_L		0xFD0D
#define IR_RX_C_COUNT_H		0xFD0E

#define IR_SUSPEND_CTRL		0xFD10

#define IR_Err_Tolerance_CTRL	0xFD11

#define IR_UNIT_LEN		0xFD12

#define IR_Err_Tolerance_LEN	0xFD13

#define IR_MAX_H_Tolerance_LEN	0xFD14
#define IR_MAX_L_Tolerance_LEN	0xFD15

#define IR_MASK_CTRL		0xFD16
#define IR_MASK_DATA		0xFD17

#define IR_RESUME_MASK_ADDR	0xFD18
#define IR_RESUME_MASK_T_LEN	0xFD19

/* DVB USB Driver stuff */
#define	USB_VID_REALTEK			0x0bda
#define	USB_PID_RTD2831U		0x2831
#define	USB_PID_RTD2838U		0x2838

#define	USB_VID_REALTEK_2		0x2304

#define	USB_PID_RTD2831U_2		0x022b
#define	USB_VID_HAIHUA			0x13d3
#define	USB_PID_HAIHUAUSB		0x3216
#define	USB_PID_HAIHUAMIN		0x3220
#define	USB_PID_HAIHUAA			0x3236
#define	USB_PID_HAIHUAL			0x3238
#define	USB_PID_HAIHUAZ			0x3244
#define	USB_VID_BILLIONTON		0x08dd
#define	USB_PID_BILLIONTON		0x2103
#define	USB_PID_COMPRO			0x0100
#define USB_PID_COMPRO_VIDEOMATE_U90	0x0150
#define	USB_VID_VESTEL			0x1a46
#define	USB_PID_VESTEL			0x1601
#define	USB_VID_FREECOM			0x14aa
#define	USB_PID_FREECOM			0x0160
#define	USB_VID_FREECOM_2		0x14ff
#define	USB_PID_FREECOM_2		0x0225

#define INFO_DEBUG 0x01
#define REG_DEBUG  0x02
#define I2C_DEBUG  0x04
#define XFER_DEBUG 0x08


#define deb_info(args...)   dprintk(r2832u_debug, INFO_DEBUG, args)
#define deb_reg(args...)    dprintk(r2832u_debug, REG_DEBUG, args)
#define deb_i2c(args...)    dprintk(r2832u_debug, I2C_DEBUG, args)
#define deb_xfer(args...)   dprintk(r2832u_debug, XFER_DEBUG, args)



#define RTL2831U_USB_TIMEOUT 1000

#define GPIO_IN 0
#define GPIO_OUT 1

enum rtl2832u_demods {
	RTL2832_DEMOD = 0x01,
	RTL2836_DEMOD = 0x02,
	RTL2840_DEMOD = 0x04
};

struct r2832u_priv {
	enum rtl2832u_demods supported_demods;
	bool high_speed;
};

#endif
